@@ -4,6 +4,7 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import re
+import sys
 from collections import defaultdict
 from datetime import datetime, timedelta, timezone
 
@@ -522,6 +523,26 @@ class couple_delta_creation_date(couple_bug_feature):
         return delta / timedelta(days=1)
 
 
+class couple_common_words_summary(couple_bug_feature):
+    def __init__(self, to_ignore=set()):
+        self.to_ignore = to_ignore
+
+    def __call__(self, bugs):
+        return set(bugs[0]["summary"].split()).intersection(
+            set(bugs[1]["summary"].split())
+        )
+
+
+class couple_common_words_comments(couple_bug_feature):
+    def __init__(self, to_ignore=set()):
+        self.to_ignore = to_ignore
+
+    def __call__(self, bugs):
+        text1 = " ".join(comment["text"] for comment in bugs[0]["comments"])
+        text2 = " ".join(comment["text"] for comment in bugs[1]["comments"])
+        return set(text1.split()).intersection(set(text2.split()))
+
+
 class couple_common_keywords(couple_bug_feature):
     def __init__(self, to_ignore=set()):
         self.to_ignore = to_ignore
@@ -621,9 +642,9 @@ class BugExtractor(BaseEstimator, TransformerMixin):
                 if res is None:
                     continue
 
-                if isinstance(res, list):
+                if isinstance(res, list) or isinstance(res, set):
                     for item in res:
-                        data[f"{item} in {feature_extractor_name}"] = "True"
+                        data[sys.intern(f"{item} in {feature_extractor_name}")] = "True"
                     continue
 
                 if isinstance(res, bool):
@@ -642,18 +663,17 @@ class BugExtractor(BaseEstimator, TransformerMixin):
 
                 # TODO: Try simply using all possible fields instead of extracting features manually.
 
+                summary = bug["summary"]
+                comments = [c["text"] for c in bug["comments"]]
                 for cleanup_function in self.cleanup_functions:
-                    bug["summary"] = cleanup_function(bug["summary"])
-                    for c in bug["comments"]:
-                        c["text"] = cleanup_function(c["text"])
+                    summary = cleanup_function(summary)
+                    comments = [cleanup_function(comment) for comment in comments]
 
                 return {
                     "data": data,
-                    "title": bug["summary"],
-                    "first_comment": ""
-                    if len(bug["comments"]) == 0
-                    else bug["comments"][0]["text"],
-                    "comments": " ".join([c["text"] for c in bug["comments"]]),
+                    "title": summary,
+                    "first_comment": "" if len(comments) == 0 else comments[0],
+                    "comments": " ".join(comments),
                 }
 
         for bug in bugs():
