@@ -4,7 +4,9 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import csv
+import re
 from datetime import datetime
+from typing import Dict, Iterator, List, NewType, Optional
 
 import tenacity
 from dateutil.relativedelta import relativedelta
@@ -13,6 +15,8 @@ from tqdm import tqdm
 
 from bugbug import db, utils
 
+BugDict = NewType("BugDict", dict)
+
 BUGS_DB = "data/bugs.json"
 db.register(
     BUGS_DB,
@@ -60,7 +64,7 @@ COMMENT_INCLUDE_FIELDS = ["id", "count", "text", "author", "creation_time"]
 PRODUCT_COMPONENT_CSV_REPORT_URL = "https://bugzilla.mozilla.org/report.cgi"
 
 
-def get_bugs(include_invalid=False):
+def get_bugs(include_invalid: Optional[bool] = False) -> Iterator[BugDict]:
     yield from (
         bug
         for bug in db.read(BUGS_DB)
@@ -165,17 +169,15 @@ def download_bugs(bug_ids, products=None, security=False):
     for bug in get_bugs(include_invalid=True):
         old_bug_count += 1
         if int(bug["id"]) in new_bug_ids:
-            new_bug_ids.remove(bug["id"])
+            new_bug_ids.remove(int(bug["id"]))
 
     print(f"Loaded {old_bug_count} bugs.")
 
     new_bug_ids = sorted(list(new_bug_ids))
 
-    CHUNK_SIZE = 100
-
     chunks = (
-        new_bug_ids[i : (i + CHUNK_SIZE)]
-        for i in range(0, len(new_bug_ids), CHUNK_SIZE)
+        new_bug_ids[i : (i + Bugzilla.BUGZILLA_CHUNK_SIZE)]
+        for i in range(0, len(new_bug_ids), Bugzilla.BUGZILLA_CHUNK_SIZE)
     )
 
     @tenacity.retry(
@@ -202,6 +204,52 @@ def download_bugs(bug_ids, products=None, security=False):
             db.append(BUGS_DB, new_bugs)
 
 
+def _find_linked(
+    bug_map: Dict[int, BugDict], bug: BugDict, link_type: str
+) -> List[int]:
+    return sum(
+        (
+            _find_linked(bug_map, bug_map[b], link_type)
+            for b in bug[link_type]
+            if b in bug_map
+        ),
+        [b for b in bug[link_type] if b in bug_map],
+    )
+
+
+def find_blocked_by(bug_map: Dict[int, BugDict], bug: BugDict) -> List[int]:
+    return _find_linked(bug_map, bug, "blocks")
+
+
+def find_blocking(bug_map: Dict[int, BugDict], bug: BugDict) -> List[int]:
+    return _find_linked(bug_map, bug, "depends_on")
+
+
+def get_fixed_versions(bug):
+    versions = set()
+
+    target_milestone_patterns = [
+        re.compile("mozilla([0-9]+)"),
+        re.compile("([0-9]+) Branch"),
+        re.compile("Firefox ([0-9]+)"),
+    ]
+    for target_milestone_pattern in target_milestone_patterns:
+        m = target_milestone_pattern.match(bug["target_milestone"])
+        if m:
+            versions.add(int(m.group(1)))
+
+    status_pattern = re.compile("cf_status_firefox([0-9]+)")
+    for field, value in bug.items():
+        if value != "fixed":
+            continue
+
+        m = status_pattern.match(field)
+        if m:
+            versions.add(int(m.group(1)))
+
+    return list(versions)
+
+
 def delete_bugs(match):
     db.delete(BUGS_DB, match)
 
@@ -218,14 +266,18 @@ def count_bugs(bug_query_params):
     return count
 
 
-def get_product_component_csv_report():
-    six_month_ago = datetime.utcnow() - relativedelta(months=6)
+def get_product_component_count(months: int = 12) -> Dict[str, int]:
+    """Returns a dictionary where keys are full components (in the form of
+    `{product}::{component}`) and the value of the number of bugs for the
+    given full components. Full component with 0 bugs are returned.
+    """
+    since = datetime.utcnow() - relativedelta(months=months)
 
     # Base params
-    url_params = {
+    params = {
         "f1": "creation_ts",
         "o1": "greaterthan",
-        "v1": six_month_ago.strftime("%Y-%m-%d"),
+        "v1": since.strftime("%Y-%m-%d"),
         "x_axis_field": "product",
         "y_axis_field": "component",
         "action": "wrap",
@@ -233,16 +285,9 @@ def get_product_component_csv_report():
         "format": "table",
     }
 
-    return PRODUCT_COMPONENT_CSV_REPORT_URL, url_params
-
-
-def get_product_component_count():
-    """Returns a dictionary where keys are full components (in the form of
-    `{product}::{component}`) and the value of the number of bugs for the
-    given full components. Full component with 0 bugs are returned.
-    """
-    url, params = get_product_component_csv_report()
-    csv_file = utils.get_session("bugzilla").get(url, params=params)
+    csv_file = utils.get_session("bugzilla").get(
+        PRODUCT_COMPONENT_CSV_REPORT_URL, params=params
+    )
     csv_file.raise_for_status()
     content = csv_file.text
 
