@@ -142,26 +142,42 @@ class source_code_file_metrics(object):
             "Average number of unique operators": commit["average_halstead_n1"],
             "Average number of operators": commit["average_halstead_N1"],
             "Average number of source loc": commit["average_source_loc"],
+            "Average number of instruction loc": commit["average_instruction_loc"],
             "Average number of logical loc": commit["average_logical_loc"],
+            "Average number of comment loc": commit["average_comment_loc"],
+            "Average number of function arguments": commit["average_nargs"],
+            "Average number of function exit points": commit["average_nexits"],
             "Maximum cyclomatic": commit["maximum_cyclomatic"],
             "Maximum number of unique operands": commit["maximum_halstead_n2"],
             "Maximum number of operands": commit["maximum_halstead_N2"],
             "Maximum number of unique operators": commit["maximum_halstead_n1"],
             "Maximum number of operators": commit["maximum_halstead_N1"],
             "Maximum number of source loc": commit["maximum_source_loc"],
+            "Maximum number of instruction loc": commit["maximum_instruction_loc"],
             "Maximum number of logical loc": commit["maximum_logical_loc"],
+            "Maximum number of comment loc": commit["maximum_comment_loc"],
+            "Maximum number of function arguments": commit["maximum_nargs"],
+            "Maximum number of function exit points": commit["maximum_nexits"],
             "Minimum cyclomatic": commit["minimum_cyclomatic"],
             "Minimum number of unique operands": commit["minimum_halstead_n2"],
             "Minimum number of operands": commit["minimum_halstead_N2"],
             "Minimum number of unique operators": commit["minimum_halstead_n1"],
             "Minimum number of operators": commit["minimum_halstead_N1"],
             "Minimum number of source loc": commit["minimum_source_loc"],
+            "Minimum number of instruction loc": commit["minimum_instruction_loc"],
             "Minimum number of logical loc": commit["minimum_logical_loc"],
+            "Minimum number of comment loc": commit["minimum_comment_loc"],
+            "Minimum number of function arguments": commit["minimum_nargs"],
+            "Minimum number of function exit points": commit["minimum_nexits"],
             "Total of number of operands": commit["total_halstead_N2"],
             "Total of number of unique operators": commit["total_halstead_n1"],
             "Total number of operators": commit["total_halstead_N1"],
             "Total number of source loc": commit["total_source_loc"],
+            "Total number of instruction loc": commit["total_instruction_loc"],
             "Total number of logical loc": commit["total_logical_loc"],
+            "Total number of comment loc": commit["total_comment_loc"],
+            "Total number of function arguments": commit["total_nargs"],
+            "Total number of function exit points": commit["total_nexits"],
         }
 
 
@@ -558,10 +574,22 @@ def merge_commits(commits: Sequence[repository.CommitDict]) -> repository.Commit
                 commit["average_source_loc"] for commit in commits
             )
             / len(commits),
+            "average_instruction_loc": sum(
+                commit["average_instruction_loc"] for commit in commits
+            )
+            / len(commits),
             "average_logical_loc": sum(
                 commit["average_logical_loc"] for commit in commits
             )
             / len(commits),
+            "average_comment_loc": sum(
+                commit["average_comment_loc"] for commit in commits
+            )
+            / len(commits),
+            "average_nargs": sum(commit["average_nargs"] for commit in commits)
+            / len(commits),
+            "average_nexits": sum(commit["average_nexits"] for commit in commits)
+            / len(commits),
             "maximum_cyclomatic": max(
                 commit["maximum_cyclomatic"] for commit in commits
             ),
@@ -580,9 +608,17 @@ def merge_commits(commits: Sequence[repository.CommitDict]) -> repository.Commit
             "maximum_source_loc": max(
                 commit["maximum_source_loc"] for commit in commits
             ),
+            "maximum_instruction_loc": max(
+                commit["maximum_instruction_loc"] for commit in commits
+            ),
             "maximum_logical_loc": max(
                 commit["maximum_logical_loc"] for commit in commits
             ),
+            "maximum_comment_loc": max(
+                commit["maximum_comment_loc"] for commit in commits
+            ),
+            "maximum_nargs": max(commit["maximum_nargs"] for commit in commits),
+            "maximum_nexits": max(commit["maximum_nexits"] for commit in commits),
             "minimum_cyclomatic": min(
                 commit["minimum_cyclomatic"] for commit in commits
             ),
@@ -601,15 +637,29 @@ def merge_commits(commits: Sequence[repository.CommitDict]) -> repository.Commit
             "minimum_source_loc": min(
                 commit["minimum_source_loc"] for commit in commits
             ),
+            "minimum_instruction_loc": min(
+                commit["minimum_instruction_loc"] for commit in commits
+            ),
             "minimum_logical_loc": min(
                 commit["minimum_logical_loc"] for commit in commits
             ),
+            "minimum_comment_loc": min(
+                commit["minimum_comment_loc"] for commit in commits
+            ),
+            "minimum_nargs": min(commit["minimum_nargs"] for commit in commits),
+            "minimum_nexits": min(commit["minimum_nexits"] for commit in commits),
             "total_halstead_n2": sum(commit["total_halstead_n2"] for commit in commits),
             "total_halstead_N2": sum(commit["total_halstead_N2"] for commit in commits),
             "total_halstead_n1": sum(commit["total_halstead_n1"] for commit in commits),
             "total_halstead_N1": sum(commit["total_halstead_N1"] for commit in commits),
             "total_source_loc": sum(commit["total_source_loc"] for commit in commits),
+            "total_instruction_loc": sum(
+                commit["total_instruction_loc"] for commit in commits
+            ),
             "total_logical_loc": sum(commit["total_logical_loc"] for commit in commits),
+            "total_comment_loc": sum(commit["total_comment_loc"] for commit in commits),
+            "total_nargs": sum(commit["total_nargs"] for commit in commits),
+            "total_nexits": sum(commit["total_nexits"] for commit in commits),
         }
     )
 
@@ -675,12 +725,10 @@ class CommitExtractor(BaseEstimator, TransformerMixin):
 
             # TODO: Try simply using all possible fields instead of extracting features manually.
 
-            for cleanup_function in self.cleanup_functions:
-                commit["desc"] = cleanup_function(commit["desc"])
-
             result = {"data": data}
             if "desc" in commit:
-                result["desc"] = commit["desc"]
+                for cleanup_function in self.cleanup_functions:
+                    result["desc"] = cleanup_function(commit["desc"])
 
             results.append(result)
 
