@@ -4,7 +4,7 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 from collections import defaultdict
-from typing import Dict, List, Tuple
+from typing import Any, Dict, List, Tuple
 
 import joblib
 import matplotlib
@@ -195,6 +195,9 @@ class Model:
             elif type_ == "data":
                 if " in " in feature_name and feature_name.endswith("=True"):
                     feature_name = feature_name[: -len("=True")]
+            elif type_ == "couple_data":
+                if " in " in feature_name and feature_name.endswith("=True"):
+                    feature_name = feature_name[: -len("=True")]
             else:
                 raise Exception(f"Unexpected feature type for: {full_feature_name}")
 
@@ -336,6 +339,10 @@ class Model:
         """Subclasses can implement their own additional evaluation."""
         pass
 
+    def get_labels(self) -> Tuple[Dict[Any, Any], List[Any]]:
+        """Subclasses implement their own function to gather labels."""
+        pass
+
     def train(self, importance_cutoff=0.15, limit=None):
         classes, self.class_names = self.get_labels()
         self.class_names = sort_class_names(self.class_names)
@@ -567,7 +574,7 @@ class Model:
         return tracking_metrics
 
     @staticmethod
-    def load(model_file_name):
+    def load(model_file_name: str) -> "Model":
         return joblib.load(model_file_name)
 
     def overwrite_classes(self, items, classes, probabilities):
@@ -711,13 +718,11 @@ class CommitModel(Model):
                 if commit["node"] in classes
             )
 
-            bug_map = {}
-
-            for bug in bugzilla.get_bugs():
-                if bug["id"] not in all_bug_ids:
-                    continue
-
-                bug_map[bug["id"]] = bug
+            bug_map = {
+                bug["id"]: bug
+                for bug in bugzilla.get_bugs()
+                if bug["id"] in all_bug_ids
+            }
 
             assert len(bug_map) > 0
 
