@@ -12,6 +12,7 @@ import logging
 import math
 import os
 import pickle
+import re
 import shelve
 import subprocess
 import sys
@@ -21,8 +22,8 @@ from functools import lru_cache
 from typing import (
     Collection,
     Dict,
-    Generator,
     Iterable,
+    Iterator,
     List,
     NewType,
     Optional,
@@ -38,7 +39,7 @@ import tenacity
 from tqdm import tqdm
 
 from bugbug import db, rust_code_analysis_server, utils
-from bugbug.utils import LMDBDict
+from bugbug.utils import LMDBDict, zstd_decompress
 
 logger = logging.getLogger(__name__)
 
@@ -55,12 +56,16 @@ COMMIT_EXPERIENCES_DB = "commit_experiences.lmdb.tar.zst"
 db.register(
     COMMITS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.data_commits.latest/artifacts/public/commits.json.zst",
-    14,
+    16,
     [COMMIT_EXPERIENCES_DB],
 )
 
+commit_to_coverage = None
 path_to_component = None
 
+HG = None
+REPO_DIR = None
+
 EXPERIENCE_TIMESPAN = 90
 EXPERIENCE_TIMESPAN_TEXT = f"{EXPERIENCE_TIMESPAN}_days"
 
@@ -108,6 +113,10 @@ TYPES_TO_EXT = {**SOURCE_CODE_TYPES_TO_EXT, **OTHER_TYPES_TO_EXT}
 
 EXT_TO_TYPES = {ext: typ for typ, exts in TYPES_TO_EXT.items() for ext in exts}
 
+PHABRICATOR_REVISION_REGEX = re.compile(
+    "Differential Revision: (https://phabricator.services.mozilla.com/D([0-9]+))"
+)
+
 
 def get_type(path: str) -> str:
     file_name = os.path.basename(path)
@@ -172,28 +181,44 @@ class Commit:
         self.average_halstead_N2 = 0.0
         self.average_halstead_n2 = 0.0
         self.average_source_loc = 0.0
+        self.average_instruction_loc = 0.0
         self.average_logical_loc = 0.0
+        self.average_comment_loc = 0.0
+        self.average_nargs = 0.0
+        self.average_nexits = 0.0
         self.maximum_cyclomatic = 0
         self.maximum_halstead_N2 = 0
         self.maximum_halstead_n2 = 0
         self.maximum_halstead_N1 = 0
         self.maximum_halstead_n1 = 0
         self.maximum_source_loc = 0
+        self.maximum_instruction_loc = 0
         self.maximum_logical_loc = 0
+        self.maximum_comment_loc = 0
+        self.maximum_nargs = 0
+        self.maximum_nexits = 0
         self.minimum_cyclomatic = sys.maxsize
         self.minimum_halstead_N1 = sys.maxsize
         self.minimum_halstead_n1 = sys.maxsize
         self.minimum_halstead_N2 = sys.maxsize
         self.minimum_halstead_n2 = sys.maxsize
         self.minimum_source_loc = sys.maxsize
+        self.minimum_instruction_loc = sys.maxsize
         self.minimum_logical_loc = sys.maxsize
+        self.minimum_comment_loc = sys.maxsize
+        self.minimum_nargs = sys.maxsize
+        self.minimum_nexits = sys.maxsize
         self.total_cyclomatic = 0
         self.total_halstead_N1 = 0
         self.total_halstead_n1 = 0
         self.total_halstead_N2 = 0
         self.total_halstead_n2 = 0
         self.total_source_loc = 0
+        self.total_instruction_loc = 0
         self.total_logical_loc = 0
+        self.total_comment_loc = 0
+        self.total_nargs = 0
+        self.total_nexits = 0
 
     def __eq__(self, other):
         assert isinstance(other, Commit)
@@ -263,7 +288,7 @@ def filter_commits(
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> Generator[CommitDict, None, None]:
+) -> Iterator[CommitDict]:
     for commit in commits:
         if not include_ignored and commit["ignored"]:
             continue
@@ -281,7 +306,7 @@ def get_commits(
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> Generator[CommitDict, None, None]:
+) -> Iterator[CommitDict]:
     return filter_commits(
         db.read(COMMITS_DB),
         include_no_bug=include_no_bug,
@@ -290,14 +315,21 @@ def get_commits(
     )
 
 
-def _init_process(repo_dir):
+def get_revision_id(commit: CommitDict) -> Optional[int]:
+    match = PHABRICATOR_REVISION_REGEX.search(commit["desc"])
+    if not match:
+        return None
+    return int(match.group(2))
+
+
+def _init_process(repo_dir: str) -> None:
     global HG, REPO_DIR
     REPO_DIR = repo_dir
     HG = hglib.open(REPO_DIR)
     get_component_mapping()
 
 
-def _init_thread(repo_dir):
+def _init_thread(repo_dir: str) -> None:
     hg_server = hglib.open(repo_dir)
     thread_local.hg = hg_server
     with hg_servers_lock:
@@ -305,7 +337,7 @@ def _init_thread(repo_dir):
 
 
 # This code was adapted from https://github.com/mozsearch/mozsearch/blob/2e24a308bf66b4c149683bfeb4ceeea3b250009a/router/router.py#L127
-def is_test(path):
+def is_test(path: str) -> bool:
     return (
         "/test/" in path
         or "/tests/" in path
@@ -432,16 +464,20 @@ def get_metrics(commit, metrics_space):
 
     if metrics_space["kind"] == "function" and not error:
         metrics = metrics_space["metrics"]
-        commit.total_cyclomatic += metrics["cyclomatic"]
+        commit.total_cyclomatic += metrics["cyclomatic"]["sum"]
         commit.total_halstead_n2 += metrics["halstead"]["n2"]
         commit.total_halstead_N2 += metrics["halstead"]["N2"]
         commit.total_halstead_n1 += metrics["halstead"]["n1"]
         commit.total_halstead_N1 += metrics["halstead"]["N1"]
         commit.total_source_loc += metrics["loc"]["sloc"]
+        commit.total_instruction_loc += metrics["loc"]["ploc"]
         commit.total_logical_loc += metrics["loc"]["lloc"]
+        commit.total_comment_loc += metrics["loc"]["cloc"]
+        commit.total_nargs += metrics["nargs"]
+        commit.total_nexits += metrics["nexits"]
 
         commit.maximum_cyclomatic = max(
-            commit.maximum_cyclomatic, metrics["cyclomatic"]
+            commit.maximum_cyclomatic, metrics["cyclomatic"]["sum"]
         )
         commit.maximum_halstead_n2 = max(
             commit.maximum_halstead_n2,
@@ -460,12 +496,20 @@ def get_metrics(commit, metrics_space):
         commit.maximum_source_loc = max(
             metrics["loc"]["sloc"], commit.maximum_source_loc
         )
+        commit.maximum_instruction_loc = max(
+            metrics["loc"]["ploc"], commit.maximum_instruction_loc
+        )
         commit.maximum_logical_loc = max(
             metrics["loc"]["lloc"], commit.maximum_logical_loc
         )
+        commit.maximum_comment_loc = max(
+            metrics["loc"]["cloc"], commit.maximum_comment_loc
+        )
+        commit.maximum_nargs = max(metrics["nargs"], commit.maximum_nargs)
+        commit.maximum_nexits = max(metrics["nexits"], commit.maximum_nexits)
 
         commit.minimum_cyclomatic = min(
-            commit.minimum_cyclomatic, metrics["cyclomatic"]
+            commit.minimum_cyclomatic, metrics["cyclomatic"]["sum"]
         )
         commit.minimum_halstead_n2 = min(
             commit.minimum_halstead_n2,
@@ -484,9 +528,17 @@ def get_metrics(commit, metrics_space):
         commit.minimum_source_loc = min(
             metrics["loc"]["sloc"], commit.minimum_source_loc
         )
+        commit.minimum_instruction_loc = min(
+            metrics["loc"]["ploc"], commit.minimum_instruction_loc
+        )
         commit.minimum_logical_loc = min(
             metrics["loc"]["lloc"], commit.minimum_logical_loc
         )
+        commit.minimum_comment_loc = min(
+            metrics["loc"]["cloc"], commit.minimum_comment_loc
+        )
+        commit.minimum_nargs = min(metrics["nargs"], commit.minimum_nargs)
+        commit.minimum_nexits = min(metrics["nexits"], commit.minimum_nexits)
 
     for space in metrics_space["spaces"]:
         error |= get_metrics(commit, space)
@@ -622,7 +674,13 @@ def transform(hg: hglib.client, repo_dir: str, commit: Commit):
         commit.average_halstead_n1 = commit.total_halstead_n1 / metrics_file_count
         commit.average_halstead_N1 = commit.total_halstead_N1 / metrics_file_count
         commit.average_source_loc = commit.total_source_loc / metrics_file_count
+        commit.average_instruction_loc = (
+            commit.total_instruction_loc / metrics_file_count
+        )
         commit.average_logical_loc = commit.total_logical_loc / metrics_file_count
+        commit.average_comment_loc = commit.total_comment_loc / metrics_file_count
+        commit.average_nargs = commit.total_nargs / metrics_file_count
+        commit.average_nexits = commit.total_nexits / metrics_file_count
     else:
         # these values are initialized with sys.maxsize (because we take the min)
         # if no files, then reset them to 0 (it'd be stupid to have min > max)
@@ -632,7 +690,11 @@ def transform(hg: hglib.client, repo_dir: str, commit: Commit):
         commit.minimum_halstead_N1 = 0
         commit.minimum_halstead_n1 = 0
         commit.minimum_source_loc = 0
+        commit.minimum_instruction_loc = 0
         commit.minimum_logical_loc = 0
+        commit.minimum_comment_loc = 0
+        commit.minimum_nargs = 0
+        commit.minimum_nexits = 0
 
     return commit
 
@@ -641,9 +703,9 @@ def _transform(commit):
     return transform(HG, REPO_DIR, commit)
 
 
-def hg_log(hg: hglib.client, revs: List[bytes]) -> List[Commit]:
+def hg_log(hg: hglib.client, revs: List[bytes]) -> Tuple[Commit, ...]:
     if len(revs) == 0:
-        return []
+        return tuple()
 
     template = "{node}\\0{author}\\0{desc}\\0{bug}\\0{backedoutby}\\0{author|email}\\0{pushdate|hgdate}\\0{reviewers}\\0{backsoutnodes}\\0"
 
@@ -694,10 +756,10 @@ def hg_log(hg: hglib.client, revs: List[bytes]) -> List[Commit]:
             )
         )
 
-    return commits
+    return tuple(commits)
 
 
-def _hg_log(revs: List[bytes]) -> List[Commit]:
+def _hg_log(revs: List[bytes]) -> Tuple[Commit, ...]:
     return hg_log(thread_local.hg, revs)
 
 
@@ -938,7 +1000,9 @@ def calculate_experiences(
             update_complex_experiences("component", day, commit.components)
 
 
-def set_commits_to_ignore(hg: hglib.client, repo_dir: str, commits: List[Commit]):
+def set_commits_to_ignore(
+    hg: hglib.client, repo_dir: str, commits: Iterable[Commit]
+) -> None:
     # Skip commits which are in .hg-annotate-ignore-revs or which have
     # 'ignore-this-changeset' in their description (mostly consisting of very
     # large and not meaningful formatting changes).
@@ -953,6 +1017,63 @@ def set_commits_to_ignore(hg: hglib.client, repo_dir: str, commits: List[Commit]
         )
 
 
+def download_coverage_mapping() -> None:
+    commit_to_coverage = get_coverage_mapping(False)
+
+    utils.download_check_etag(
+        "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/project.relman.code-coverage.production.cron.latest/artifacts/public/commit_coverage.json.zst",
+        "data/coverage_mapping.json.zst",
+    )
+
+    zstd_decompress("data/coverage_mapping.json")
+    assert os.path.exists("data/coverage_mapping.json")
+
+    with open("data/coverage_mapping.json", "r") as f:
+        data = json.load(f)
+
+    for commit_hash, commit_stats in data.items():
+        commit_to_coverage[commit_hash.encode("utf-8")] = pickle.dumps(commit_stats)
+
+    close_coverage_mapping()
+
+
+def get_coverage_mapping(readonly: bool = True) -> LMDBDict:
+    global commit_to_coverage
+    commit_to_coverage = LMDBDict("data/coverage_mapping.lmdb", readonly=readonly)
+    return commit_to_coverage
+
+
+def close_coverage_mapping() -> None:
+    global commit_to_coverage
+    assert commit_to_coverage is not None
+    commit_to_coverage.close()
+    commit_to_coverage = None
+
+
+def set_commit_coverage(commits: Iterable[CommitDict]) -> None:
+    commit_to_coverage = get_coverage_mapping(True)
+
+    for commit in commits:
+        try:
+            commit_stats = pickle.loads(
+                commit_to_coverage[commit["node"].encode("utf-8")]
+            )
+            if commit_stats is not None:
+                added = commit_stats["added"]
+                covered = commit_stats["covered"]
+                unknown = commit_stats["unknown"]
+            else:
+                added = covered = unknown = None
+        except KeyError:
+            added = covered = unknown = None
+
+        commit["cov_added"] = added
+        commit["cov_covered"] = covered
+        commit["cov_unknown"] = unknown
+
+    close_coverage_mapping()
+
+
 def download_component_mapping():
     path_to_component = get_component_mapping(False)
 
@@ -982,11 +1103,12 @@ def close_component_mapping():
     path_to_component = None
 
 
-def hg_log_multi(repo_dir, revs):
+def hg_log_multi(repo_dir: str, revs: List[bytes]) -> Tuple[Commit, ...]:
     if len(revs) == 0:
-        return []
+        return tuple()
 
-    threads_num = os.cpu_count() + 1
+    cpu_count = os.cpu_count()
+    threads_num = cpu_count + 1 if cpu_count is not None else 1
     REVS_COUNT = len(revs)
     CHUNK_SIZE = int(math.ceil(REVS_COUNT / threads_num))
     revs_groups = [revs[i : i + CHUNK_SIZE] for i in range(0, REVS_COUNT, CHUNK_SIZE)]
@@ -994,9 +1116,9 @@ def hg_log_multi(repo_dir, revs):
     with concurrent.futures.ThreadPoolExecutor(
         initializer=_init_thread, initargs=(repo_dir,), max_workers=threads_num
     ) as executor:
-        commits = executor.map(_hg_log, revs_groups)
-        commits = tqdm(commits, total=len(revs_groups))
-        commits = list(itertools.chain.from_iterable(commits))
+        commits_iter = executor.map(_hg_log, revs_groups)
+        commits_iter = tqdm(commits_iter, total=len(revs_groups))
+        commits = tuple(itertools.chain.from_iterable(commits_iter))
 
     while len(hg_servers) > 0:
         hg_server = hg_servers.pop()
@@ -1045,6 +1167,10 @@ def download_commits(
             logger.info("Downloading file->component mapping...")
             download_component_mapping()
 
+        if save or not os.path.exists("data/coverage_mapping.lmdb"):
+            logger.info("Downloading commit->coverage mapping...")
+            download_coverage_mapping()
+
         set_commits_to_ignore(hg, repo_dir, commits)
 
         commits_num = len(commits)
@@ -1058,9 +1184,9 @@ def download_commits(
             with concurrent.futures.ProcessPoolExecutor(
                 initializer=_init_process, initargs=(repo_dir,)
             ) as executor:
-                commits = executor.map(_transform, commits, chunksize=64)
-                commits = tqdm(commits, total=commits_num)
-                commits = tuple(commits)
+                commits_iter = executor.map(_transform, commits, chunksize=64)
+                commits_iter = tqdm(commits_iter, total=commits_num)
+                commits = tuple(commits_iter)
         else:
             get_component_mapping()
 
@@ -1074,14 +1200,16 @@ def download_commits(
 
     logger.info("Applying final commits filtering...")
 
-    commits = tuple(commit.to_dict() for commit in commits)
+    commit_dicts = tuple(commit.to_dict() for commit in commits)
+
+    set_commit_coverage(commit_dicts)
 
     if save:
-        db.append(COMMITS_DB, commits)
+        db.append(COMMITS_DB, commit_dicts)
 
     return tuple(
         filter_commits(
-            commits,
+            commit_dicts,
             include_no_bug=include_no_bug,
             include_backouts=include_backouts,
             include_ignored=include_ignored,
@@ -1089,6 +1217,18 @@ def download_commits(
     )
 
 
+def update_commits() -> None:
+    commits = list(
+        get_commits(include_no_bug=True, include_backouts=True, include_ignored=True)
+    )
+
+    # Add coverage information for previous commits too.
+    # We need to do this because coverage information is sometimes slow to come in.
+    set_commit_coverage(commits)
+
+    db.write(COMMITS_DB, commits)
+
+
 def clean(hg, repo_dir):
     logger.info("Restoring files to their checkout state...")
     hg.revert(repo_dir.encode("utf-8"), all=True)
