@@ -29,21 +29,21 @@ OPT_MSG_MISSING = (
 )
 
 try:
-    import nltk
     import gensim
+    import nltk
+    import spacy
+    from elasticsearch import Elasticsearch
+    from elasticsearch.helpers import bulk
     from gensim import models, similarities
-    from gensim.models import Word2Vec, WordEmbeddingSimilarityIndex, TfidfModel
-    from gensim.models.ldamodel import LdaModel
+    from gensim.corpora import Dictionary
     from gensim.matutils import sparse2full
+    from gensim.models import TfidfModel, Word2Vec, WordEmbeddingSimilarityIndex
+    from gensim.models.ldamodel import LdaModel
     from gensim.similarities import SoftCosineSimilarity, SparseTermSimilarityMatrix
     from gensim.summarization.bm25 import BM25
-    from gensim.corpora import Dictionary
-    from elasticsearch.helpers import bulk
-    from elasticsearch import Elasticsearch
     from nltk.corpus import stopwords
     from nltk.stem.porter import PorterStemmer
     from nltk.tokenize import word_tokenize
-    import spacy
     from wmd import WMD
 except ImportError:
     raise ImportError(OPT_MSG_MISSING)
@@ -64,7 +64,7 @@ REPORTERS_TO_IGNORE = {"intermittent-bug-filer@mozilla.bugs", "wptsync@mozilla.b
 
 def download_similarity_model(model_name):
     path = f"{model_name_to_class[model_name].__name__.lower()}.similaritymodel"
-    url = f"https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.train_similarity.latest/artifacts/public/{path}.zst"
+    url = f"https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.train_similarity.latest/artifacts/public/{path}.zst"
 
     logger.info(f"Downloading similarity model from {url}...")
     updated = download_check_etag(url)
