@@ -2,6 +2,7 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
+
 import os
 
 from bugbug import repository
@@ -143,55 +144,98 @@ class arch(object):
         return archs.pop()
 
 
+def get_manifest(runnable):
+    if isinstance(runnable, str):
+        return runnable
+    else:
+        return runnable[1]
+
+
+def commonprefix(path1, path2):
+    for i, c in enumerate(path1):
+        if c != path2[i]:
+            return path1[:i]
+    return path1
+
+
 class path_distance(object):
     def __call__(self, test_job, commit, **kwargs):
-        distances = []
-        for path in commit["files"]:
-            movement = os.path.relpath(
-                os.path.dirname(test_job["name"]), os.path.dirname(path)
-            )
-            if movement == ".":
-                distances.append(0)
-            else:
-                distances.append(movement.count("/") + 1)
+        min_distance = None
 
-        return min(distances, default=None)
+        manifest = get_manifest(test_job["name"])
+
+        for path in commit["files"]:
+            i = len(commonprefix(manifest, path))
+            distance = manifest[i:].count("/") + path[i:].count("/")
+
+            if min_distance is None or min_distance > distance:
+                min_distance = distance
+
+        return min_distance
 
 
 class common_path_components(object):
     def __call__(self, test_job, commit, **kwargs):
-        test_components = set(test_job["name"].split("/"))
-        common_components_numbers = [
+        manifest = get_manifest(test_job["name"])
+        test_components = set(manifest.split("/"))
+        common_components_numbers = (
             len(set(path.split("/")) & test_components) for path in commit["files"]
-        ]
+        )
         return max(common_components_numbers, default=None)
 
 
 class first_common_parent_distance(object):
     def __call__(self, test_job, commit, **kwargs):
-        distances = []
+        min_distance = None
+
+        manifest = get_manifest(test_job["name"])
+
         for path in commit["files"]:
             path_components = path.split("/")
 
             for i in range(len(path_components) - 1, 0, -1):
-                if test_job["name"].startswith("/".join(path_components[:i])):
-                    distances.append(i)
+                if manifest.startswith("/".join(path_components[:i])):
+                    if min_distance is None or min_distance > i:
+                        min_distance = i
                     break
 
-        return min(distances, default=None)
+        return min_distance
 
 
 class same_component(object):
     def __call__(self, test_job, commit, **kwargs):
+        manifest = get_manifest(test_job["name"])
+
         component_mapping = repository.get_component_mapping()
 
-        if test_job["name"].encode("utf-8") not in component_mapping:
+        if manifest.encode("utf-8") not in component_mapping:
             return None
 
         touches_same_component = any(
-            component_mapping[test_job["name"].encode("utf-8")]
+            component_mapping[manifest.encode("utf-8")]
             == component_mapping[f.encode("utf-8")]
             for f in commit["files"]
             if f.encode("utf-8") in component_mapping
         )
         return touches_same_component
+
+
+class manifest_suite(object):
+    def __call__(self, test_job, commit, **kwargs):
+        manifest = get_manifest(test_job["name"])
+
+        if manifest.startswith("testing/web-platform/"):
+            return "WPT"
+
+        base = os.path.basename(manifest)
+
+        if any(s in base for s in ("chrome", "browser", "mochitest", "a11y")):
+            return "mochitest"
+        elif base == "jstests.list":
+            return "jstest"
+        elif "xpcshell" in base:
+            return "xpcshell"
+        elif "reftest" in base:
+            return "reftest"
+
+        return None
