@@ -3,6 +3,7 @@
 import argparse
 from datetime import datetime
 from logging import getLogger
+from typing import List
 
 import dateutil.parser
 from dateutil.relativedelta import relativedelta
@@ -14,7 +15,7 @@ logger = getLogger(__name__)
 
 
 class Retriever(object):
-    def retrieve_bugs(self, limit=None):
+    def retrieve_bugs(self, limit=None) -> None:
         bugzilla.set_token(get_secret("BUGZILLA_TOKEN"))
 
         db.download(bugzilla.BUGS_DB)
@@ -29,6 +30,18 @@ class Retriever(object):
         )
         logger.info(f"Retrieved {len(changed_ids)} IDs.")
 
+        all_components = bugzilla.get_product_component_count(9999)
+
+        deleted_component_ids = [
+            bug["id"]
+            for bug in bugzilla.get_bugs()
+            if "{}::{}".format(bug["product"], bug["component"]) not in all_components
+        ]
+        logger.info(
+            f"{len(deleted_component_ids)} bugs belonging to deleted components"
+        )
+        changed_ids += deleted_component_ids
+
         # Get IDs of bugs between (two years and six months ago) and (six months ago).
         six_months_ago = datetime.utcnow() - relativedelta(months=6)
         two_years_and_six_months_ago = six_months_ago - relativedelta(years=2)
@@ -39,13 +52,13 @@ class Retriever(object):
             two_years_and_six_months_ago, six_months_ago
         )
         if limit:
-            timespan_ids = timespan_ids[:limit]
+            timespan_ids = timespan_ids[-limit:]
         logger.info(f"Retrieved {len(timespan_ids)} IDs.")
 
         # Get IDs of labelled bugs.
         labelled_bug_ids = labels.get_all_bug_ids()
         if limit:
-            labelled_bug_ids = labelled_bug_ids[:limit]
+            labelled_bug_ids = labelled_bug_ids[-limit:]
         logger.info(f"{len(labelled_bug_ids)} labelled bugs to download.")
 
         # Get the commits DB, as we need it to get the bug IDs linked to recent commits.
@@ -54,34 +67,40 @@ class Retriever(object):
             assert db.download(repository.COMMITS_DB)
 
         # Get IDs of bugs linked to commits (used for some commit-based models, e.g. backout and regressor).
-        start_date = datetime.now() - relativedelta(years=2, months=6)
-        commit_bug_ids = [
-            commit["bug_id"]
-            for commit in repository.get_commits()
-            if commit["bug_id"]
-            and dateutil.parser.parse(commit["pushdate"]) >= start_date
-        ]
+        start_date = datetime.now() - relativedelta(years=3)
+        commit_bug_ids = list(
+            set(
+                commit["bug_id"]
+                for commit in repository.get_commits()
+                if commit["bug_id"]
+                and dateutil.parser.parse(commit["pushdate"]) >= start_date
+            )
+        )
         if limit:
             commit_bug_ids = commit_bug_ids[-limit:]
         logger.info(f"{len(commit_bug_ids)} bugs linked to commits to download.")
 
-        # Get IDs of bugs which caused regressions fixed by commits (useful for the regressor model).
-        regressed_by_bug_ids = sum(
-            [
-                bug["regressed_by"]
-                for bug in bugzilla.get_bugs()
-                if bug["id"] in commit_bug_ids
-            ],
-            [],
+        # Get IDs of bugs which are regressions, bugs which caused regressions (useful for the regressor model),
+        # and blocked bugs.
+        regression_related_ids: List[int] = list(
+            set(
+                sum(
+                    (
+                        bug["regressed_by"] + bug["regressions"] + bug["blocks"]
+                        for bug in bugzilla.get_bugs()
+                    ),
+                    [],
+                )
+            )
         )
         if limit:
-            regressed_by_bug_ids = regressed_by_bug_ids[-limit:]
+            regression_related_ids = regression_related_ids[-limit:]
         logger.info(
-            f"{len(regressed_by_bug_ids)} bugs which caused regressions fixed by commits."
+            f"{len(regression_related_ids)} bugs which caused regressions fixed by commits."
         )
 
         all_ids = (
-            timespan_ids + labelled_bug_ids + commit_bug_ids + regressed_by_bug_ids
+            timespan_ids + labelled_bug_ids + commit_bug_ids + regression_related_ids
         )
         all_ids_set = set(all_ids)
 
@@ -93,24 +112,34 @@ class Retriever(object):
 
         bugzilla.download_bugs(all_ids)
 
-        # Get regressed_by_bug_ids again (the set could have changed after downloading new bugs).
-        regressed_by_bug_ids = sum(
-            [
-                bug["regressed_by"]
-                for bug in bugzilla.get_bugs()
-                if bug["id"] in commit_bug_ids
-            ],
-            [],
-        )
-        logger.info(
-            f"{len(regressed_by_bug_ids)} bugs which caused regressions fixed by commits."
-        )
-
-        bugzilla.download_bugs(regressed_by_bug_ids)
-
-        # Try to re-download inconsistent bugs, up to three times.
-        inconsistent_bugs = bugzilla.get_bugs(include_invalid=True)
+        # Get regression_related_ids again (the set could have changed after downloading new bugs).
         for i in range(3):
+            regression_related_ids = list(
+                set(
+                    sum(
+                        (
+                            bug["regressed_by"] + bug["regressions"] + bug["blocks"]
+                            for bug in bugzilla.get_bugs()
+                        ),
+                        [],
+                    )
+                )
+            )
+            logger.info(
+                f"{len(regression_related_ids)} bugs which caused regressions fixed by commits."
+            )
+            if limit:
+                regression_related_ids = regression_related_ids[-limit:]
+
+            # If we got all bugs we needed, break.
+            if set(regression_related_ids).issubset(all_ids):
+                break
+
+            bugzilla.download_bugs(regression_related_ids)
+
+        # Try to re-download inconsistent bugs, up to twice.
+        inconsistent_bugs = bugzilla.get_bugs(include_invalid=True)
+        for i in range(2):
             # We look for inconsistencies in all bugs first, then, on following passes,
             # we only look for inconsistencies in bugs that were found to be inconsistent in the first pass
             inconsistent_bugs = bug_snapshot.get_inconsistencies(inconsistent_bugs)
@@ -125,10 +154,19 @@ class Retriever(object):
             bugzilla.delete_bugs(lambda bug: bug["id"] in inconsistent_bug_ids)
             bugzilla.download_bugs(inconsistent_bug_ids)
 
-        zstd_compress("data/bugs.json")
+        # TODO: Figure out why.
+        missing_history_bug_ids = {
+            bug["id"] for bug in bugzilla.get_bugs() if "history" not in bug
+        }
+        bugzilla.delete_bugs(lambda bug: bug["id"] in missing_history_bug_ids)
+        logger.info(
+            f"Deleted {len(missing_history_bug_ids)} bugs as we couldn't retrieve their history"
+        )
+
+        zstd_compress(bugzilla.BUGS_DB)
 
 
-def main():
+def main() -> None:
     description = "Retrieve and extract the information from Bugzilla instance"
     parser = argparse.ArgumentParser(description=description)
     parser.add_argument(
