@@ -6,11 +6,11 @@ import csv
 import io
 import json
 import os
-import pickle
 import re
 import subprocess
 from datetime import datetime
 from logging import INFO, basicConfig, getLogger
+from typing import Optional, Tuple, cast
 
 import dateutil.parser
 import hglib
@@ -26,9 +26,11 @@ from libmozdata.phabricator import PhabricatorAPI
 from scipy.stats import spearmanr
 
 from bugbug import db, repository, test_scheduling
+from bugbug.model import Model
+from bugbug.models.testfailure import TestFailureModel
 from bugbug.utils import (
-    download_and_load_model,
     download_check_etag,
+    download_model,
     get_secret,
     to_array,
     zstd_decompress,
@@ -38,7 +40,7 @@ basicConfig(level=INFO)
 logger = getLogger(__name__)
 
 URL = "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.train_{model_name}.latest/artifacts/public/{file_name}"
-PAST_BUGS_BY_FUNCTION_URL = "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.past_bugs_by_function.latest/artifacts/public/past_bugs_by_function.pickle.zst"
+PAST_BUGS_BY_FUNCTION_URL = "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.past_bugs_by_unit.latest/artifacts/public/past_fixed_bugs_by_function.json.zst"
 PHAB_PROD = "prod"
 PHAB_DEV = "dev"
 
@@ -137,7 +139,7 @@ class CommitClassifier(object):
         self.model_name = model_name
         self.repo_dir = repo_dir
 
-        self.model = download_and_load_model(model_name)
+        self.model = Model.load(download_model(model_name))
         assert self.model is not None
 
         self.git_repo_dir = git_repo_dir
@@ -179,14 +181,14 @@ class CommitClassifier(object):
             self.X = to_array(joblib.load(model_data_X_path))
             self.y = to_array(joblib.load(model_data_y_path))
 
-            past_bugs_by_function_path = "data/past_bugs_by_function.pickle"
+            past_bugs_by_function_path = "data/past_fixed_bugs_by_function.json"
             download_check_etag(
                 PAST_BUGS_BY_FUNCTION_URL, path=f"{past_bugs_by_function_path}.zst"
             )
             zstd_decompress(past_bugs_by_function_path)
             assert os.path.exists(past_bugs_by_function_path)
-            with open(past_bugs_by_function_path, "rb") as f:
-                self.past_bugs_by_function = pickle.load(f)
+            with open(past_bugs_by_function_path, "r") as f:
+                self.past_bugs_by_function = json.load(f)
 
         if model_name == "testlabelselect":
             self.use_test_history = True
@@ -196,7 +198,9 @@ class CommitClassifier(object):
             )
             self.past_failures_data = test_scheduling.get_past_failures("label", True)
 
-            self.testfailure_model = download_and_load_model("testfailure")
+            self.testfailure_model = cast(
+                TestFailureModel, TestFailureModel.load(download_model("testfailure"))
+            )
             assert self.testfailure_model is not None
 
     def clone_git_repo(self, repo_url, repo_dir, rev="origin/branches/default/tip"):
@@ -567,11 +571,11 @@ class CommitClassifier(object):
 
     def classify(
         self,
-        revision=None,
-        phabricator_deployment=None,
-        diff_id=None,
-        runnable_jobs_path=None,
-    ):
+        revision: Optional[str] = None,
+        phabricator_deployment: Optional[str] = None,
+        diff_id: Optional[int] = None,
+        runnable_jobs_path: Optional[str] = None,
+    ) -> None:
         if revision is not None:
             assert phabricator_deployment is None
             assert diff_id is None
@@ -595,18 +599,18 @@ class CommitClassifier(object):
                 use_single_process=self.use_single_process,
             )
         else:
-            commits = []
-
-            for commit in repository.get_commits():
-                if commit["node"] == revision:
-                    commits.append(commit)
-                    break
+            assert revision is not None
+            commits = tuple(
+                commit
+                for commit in repository.get_commits()
+                if commit["node"] == revision
+            )
 
             # The commit to analyze was not in our DB, let's mine it.
             if len(commits) == 0:
                 commits = repository.download_commits(
                     self.repo_dir,
-                    revs=[revision],
+                    revs=[revision.encode("ascii")],
                     save=False,
                     use_single_process=self.use_single_process,
                 )
@@ -618,7 +622,7 @@ class CommitClassifier(object):
         else:
             self.classify_test_select(commits, runnable_jobs_path)
 
-    def classify_regressor(self, commits):
+    def classify_regressor(self, commits: Tuple[repository.CommitDict, ...]) -> None:
         # We use "clean" (or "dirty") commits as the background dataset for feature importance.
         # This way, we can see the features which are most important in differentiating
         # the current commit from the "clean" (or "dirty") commits.
@@ -780,15 +784,18 @@ class CommitClassifier(object):
                         continue
 
                     if method_level_result["method_name"].endswith(function_name):
-                        method_level_result["past_bugs"] = list(
-                            self.past_bugs_by_function[path][function_name]["bugs"]
-                        )
+                        method_level_result["past_bugs"] = [
+                            "Bug {} - {}".format(bug["id"], bug["summary"])
+                            for bug in self.past_bugs_by_function[path][function_name][
+                                -3:
+                            ]
+                        ]
 
         with open("method_level.json", "w") as f:
             json.dump(method_level_results, f)
 
 
-def main():
+def main() -> None:
     description = "Classify a commit"
     parser = argparse.ArgumentParser(description=description)
 
