@@ -13,8 +13,6 @@ logger = getLogger(__name__)
 
 class Retriever(object):
     def __init__(self, cache_root):
-        self.cache_root = cache_root
-
         assert os.path.isdir(cache_root), f"Cache root {cache_root} is not a dir."
         self.repo_dir = os.path.join(cache_root, "mozilla-central")
 
@@ -35,6 +33,10 @@ class Retriever(object):
 
         logger.info("commit data extracted from repository")
 
+        # Some commits that were already in the DB from the previous run might need
+        # to be updated (e.g. coverage information).
+        repository.update_commits()
+
         zstd_compress(repository.COMMITS_DB)
         create_tar_zst(os.path.join("data", repository.COMMIT_EXPERIENCES_DB))
 
