@@ -12,6 +12,7 @@ import time
 from collections import defaultdict
 from datetime import datetime
 from logging import INFO, basicConfig, getLogger
+from typing import Iterator, cast
 
 import dateutil.parser
 import tenacity
@@ -21,16 +22,14 @@ from microannotate import utils as microannotate_utils
 from tqdm import tqdm
 
 from bugbug import bugzilla, db, repository
+from bugbug.models.defect_enhancement_task import DefectEnhancementTaskModel
+from bugbug.models.regression import RegressionModel
 from bugbug.models.regressor import (
     BUG_FIXING_COMMITS_DB,
     BUG_INTRODUCING_COMMITS_DB,
     TOKENIZED_BUG_INTRODUCING_COMMITS_DB,
 )
-from bugbug.utils import (
-    ThreadPoolExecutorResult,
-    download_and_load_model,
-    zstd_compress,
-)
+from bugbug.utils import ThreadPoolExecutorResult, download_model, zstd_compress
 
 basicConfig(level=INFO)
 logger = getLogger(__name__)
@@ -174,7 +173,7 @@ class RegressorFinder(object):
         zstd_compress(IGNORED_COMMITS_DB)
         db.upload(IGNORED_COMMITS_DB)
 
-    def find_bug_fixing_commits(self):
+    def find_bug_fixing_commits(self) -> None:
         logger.info("Downloading commits database...")
         assert db.download(repository.COMMITS_DB)
 
@@ -185,18 +184,25 @@ class RegressorFinder(object):
         db.download(BUG_FIXING_COMMITS_DB)
 
         logger.info("Get previously classified commits...")
-        prev_bug_fixing_commits = list(db.read(BUG_FIXING_COMMITS_DB))
         prev_bug_fixing_commits_nodes = set(
-            bug_fixing_commit["rev"] for bug_fixing_commit in prev_bug_fixing_commits
+            bug_fixing_commit["rev"]
+            for bug_fixing_commit in db.read(BUG_FIXING_COMMITS_DB)
+        )
+        logger.info(
+            f"Already classified {len(prev_bug_fixing_commits_nodes)} commits..."
         )
-        logger.info(f"Already classified {len(prev_bug_fixing_commits)} commits...")
 
         # TODO: Switch to the pure Defect model, as it's better in this case.
         logger.info("Downloading defect/enhancement/task model...")
-        defect_model = download_and_load_model("defectenhancementtask")
+        defect_model = cast(
+            DefectEnhancementTaskModel,
+            DefectEnhancementTaskModel.load(download_model("defectenhancementtask")),
+        )
 
         logger.info("Downloading regression model...")
-        regression_model = download_and_load_model("regression")
+        regression_model = cast(
+            RegressionModel, RegressionModel.load(download_model("regression"))
+        )
 
         start_date = datetime.now() - RELATIVE_START_DATE
         end_date = datetime.now() - RELATIVE_END_DATE
@@ -219,7 +225,7 @@ class RegressorFinder(object):
         )
         assert len(commit_map) > 0
 
-        def get_relevant_bugs():
+        def get_relevant_bugs() -> Iterator[dict]:
             return (bug for bug in bugzilla.get_bugs() if bug["id"] in commit_map)
 
         bug_count = sum(1 for bug in get_relevant_bugs())
@@ -227,12 +233,12 @@ class RegressorFinder(object):
             f"{bug_count} bugs in total, {len(commit_map) - bug_count} bugs linked to commits missing"
         )
 
-        known_defect_labels = defect_model.get_labels()
-        known_regression_labels = regression_model.get_labels()
+        known_defect_labels, _ = defect_model.get_labels()
+        known_regression_labels, _ = regression_model.get_labels()
 
         bug_fixing_commits = []
 
-        def append_bug_fixing_commits(bug_id, type_):
+        def append_bug_fixing_commits(bug_id: int, type_: str) -> None:
             for commit in commit_map[bug_id]:
                 bug_fixing_commits.append({"rev": commit, "type": type_})
 
