@@ -115,10 +115,10 @@ class Retriever(object):
                             tuple(push.get_possible_regressions(granularity)),
                             tuple(push.get_likely_regressions(granularity)),
                         )
-                        adr.config.cache.put(
+                        mozci.config.cache.put(
                             key,
                             (value, MOZCI_VERSION),
-                            adr.config["cache"]["retention"],
+                            mozci.config["cache"]["retention"],
                         )
                         assert len(value) == 5
                         yield value
@@ -135,7 +135,7 @@ class Retriever(object):
 
         def retrieve_from_cache(push):
             semaphore.acquire()
-            return adr.config.cache.get(cache_key(push))
+            return mozci.config.cache.get(cache_key(push))
 
         with concurrent.futures.ThreadPoolExecutor() as executor:
             futures = [executor.submit(retrieve_from_cache, push) for push in pushes]
@@ -243,6 +243,15 @@ class Retriever(object):
                     skipped_no_commits += 1
                     continue
 
+                # Skip wptsync commits, since they are not like normal pushes made by developers.
+                if any(
+                    "wptsync" in commit["author_email"]
+                    or "wpt-pr:" in commit["desc"]
+                    or "wpt-type:" in commit["desc"]
+                    for commit in commits
+                ):
+                    continue
+
                 merged_commits = commit_features.merge_commits(commits)
 
                 # XXX: For now, skip commits which are too large.
